package com.blazinggames.gamejam.layers.starling
{
	import flash.utils.getTimer;
	
	import starling.core.Starling;
	import starling.display.DisplayObject;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.KeyboardEvent;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.text.TextField;
	import starling.utils.HAlign;
	
	public class ScreenManager extends Sprite
	{
// ***** CONSTANTS *****
		
		public static const GAME_OVER_EVENT:String = "GameOver";
		public static const GAME_WON_EVENT:String = "GameWon";
		private static const UPDATE_MS_CAP:int = 200;
		public static const DEFAULT_MS_PER_INTERVAL:int = 10;
		
		// Standard game states (some reserved for future needs, more can be appended)
		public static const GAMESTATE_STARTING:int = 0;
		public static const GAMESTATE_PRE_PLAY:int = 1; // reserved
		public static const GAMESTATE_PLAYING:int = 2;
		public static const GAMESTATE_PLAYING_RESERVED1:int = 3;
		public static const GAMESTATE_PLAYING_RESERVED2:int = 4;
		public static const GAMESTATE_PLAYING_RESERVED3:int = 5;		
		public static const GAMESTATE_LOST_PLAYER:int = 6;
		public static const GAMESTATE_EVENT_RESERVED1:int = 7;
		public static const GAMESTATE_EVENT_RESERVED2:int = 8;
		public static const GAMESTATE_LEVEL_COMPLETE:int = 9;
		public static const GAMESTATE_GAME_OVER:int = 10;
		public static const GAMESTATE_GAME_WON:int = 11;
		
// ***** VARIABLES *****
		
		// state and overlay control
		public var _globalData:Object;
		protected var _overlayMessage:TextField;
		protected var _gameState:int;
		
		// interval management
		protected var _lastUpdateTime:int;
		protected var _intervalRate:int;
		
		// screen management
		protected var _screens:Object;
		protected var _currentScreen:IScreen;
		protected var _screenStack:Array;
		
// ***** CONSTRUCTOR *****
				
		public function ScreenManager()
		{
			super();
			addEventListener(Event.ADDED_TO_STAGE, handleAddedToStage);
			_intervalRate = DEFAULT_MS_PER_INTERVAL;
			_screens = new Object();
			_currentScreen = null;
			_screenStack = new Array();
		}
		
		protected function handleAddedToStage(evt:Event):void
		{
			_overlayMessage = new TextField(640,150, "Hello Starling...", "_typewriter", 24, 0xFFCC00, true);
			_overlayMessage.hAlign = HAlign.CENTER;
			_overlayMessage.visible = false;
			addChild(_overlayMessage);
			addEventListener(TouchEvent.TOUCH, handleTouch);
			_lastUpdateTime = getTimer();
			addEventListener(Event.ENTER_FRAME, handleEnterFrame);
			changeGameState(GAMESTATE_STARTING);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			stage.addEventListener(KeyboardEvent.KEY_UP, handleKeyUp);
		}
				
// ***** METHODS - Overlay Management *****
		
		public function showOverlay(message:String):void
		{
			// Make sure overlay is the top layer by re-adding it
			if (contains(_overlayMessage))
				removeChild(_overlayMessage);
			
			_overlayMessage.text = message;
			addChild(_overlayMessage);
			_overlayMessage.visible = true;			
		}
		
		public function hideOverlay():void
		{
			if (contains(_overlayMessage))
				removeChild(_overlayMessage);
		}
		
// ***** METHODS - Game State Management *****
				
		protected function clearState(oldState:int):void
		{
			hideOverlay();
		}
		
		protected function startState(newState:int):void
		{
			showOverlay("GAME NOT IMPLEMENTED!\nClick to exit...\n"+Starling.context.driverInfo);
		}
		
		public function changeGameState(state:int):void
		{
			clearState(_gameState);
			_gameState = state;
			startState(_gameState);
		}
		
// ***** METHODS - event handlers *****
		
		public function handleTouch(evt:TouchEvent):void
		{
			var touch:Touch = evt.getTouch(this);
			if (touch == null) return;
			if (touch.phase == TouchPhase.ENDED)
				changeGameState(GAMESTATE_PLAYING);
		}
	
		protected function handleKeyDown(evt:KeyboardEvent):void
		{
			// TODO - Proper key flag support
			if (_currentScreen)
				_currentScreen.handleKeyboard(evt.keyCode, true, 0);
		}

		protected function handleKeyUp(evt:KeyboardEvent):void
		{
			// TODO - Proper key flag support
			if (_currentScreen)
				_currentScreen.handleKeyboard(evt.keyCode, false, 0);
		}

		protected function handleEnterFrame(event:Event):void
		{
			/*			
			if (_gameState != GAMESTATE_PLAYING) {
			_lastUpdateTime = getTimer();
			return;
			}
			*/			
			var curTime:int = getTimer();
			var delta:int = curTime - _lastUpdateTime;
			if (delta > UPDATE_MS_CAP) {
				delta = UPDATE_MS_CAP;
				_lastUpdateTime = curTime - delta;
			}
			var numIntervals:int = delta / _intervalRate;
			_lastUpdateTime += (numIntervals * _intervalRate);
			//									trace("Number of intervals is " + numIntervals);
			var startingState:int = _gameState;
			for (var cntr:int = 0; ((cntr < numIntervals) && (_gameState == startingState)); ++cntr)
				updateWorld();			
		}
		
		public function updateWorld():void
		{
			if (_currentScreen == null)
				return;
			else
				_currentScreen.update(_globalData);
		}
		
// ***** METHODS - Screen Management *****
		
		public function registerScreen(screenName:String, screen:IScreen, pushToCurrent:Boolean = false):void
		{
			_screens[screenName] = screen;
			if (pushToCurrent)
				pushScreen(screenName);
		}
		
		public function pushScreen(screenName:String):void
		{
			if (_currentScreen != null) {
				_screenStack.push(_currentScreen);
				removeChild(_currentScreen as DisplayObject);
			}
			_currentScreen = _screens[screenName];
			if (_currentScreen)
				addChild(_currentScreen as DisplayObject);
		}
		
		public function popScreen():void
		{
			if (_currentScreen)
				removeChild(_currentScreen as DisplayObject);
			_currentScreen = _screenStack.pop();
			if (_currentScreen)
				addChild(_currentScreen as DisplayObject);
		}
		
		public function swapScreen(screenName:String):void
		{
			if (_currentScreen)
				removeChild(_currentScreen as DisplayObject);
			_currentScreen = _screens[screenName];
			if (_currentScreen)
				addChild(_currentScreen as DisplayObject);
		}
	}
}