package com.blazinggames.gamejam.layers.starling
{
	import com.blazinggames.gamejam.data.TileMap;
	import com.blazinggames.gamejam.layers.ILayerRenderer;
	
	import flash.geom.Rectangle;
	
	import starling.display.Image;
	import starling.display.Sprite;
	import starling.textures.Texture;
	
	public class TileMapRenderer extends Sprite implements ILayerRenderer
	{
// ***** VARIABLES *****
		
		private var _map:TileMap;
		private var _textures:Vector.<Texture>;
		private var _viewportWidth:int, _viewportHeight:int;
		private var _tiles:Array;//[y][x] format
		private var _viewWidth:int, _viewHeight:int;
		private var _oldX:int, _oldY:int;

// ***** CONSTRUCTOR *****
		
		public function TileMapRenderer(map:TileMap, textures:Vector.<Texture>)
		{
			super();
			_map = map;
			_textures = textures;
			_tiles = new Array();
			_tiles[0] = new Array();
			_tiles[0][0] = new Image(_textures[0]);
			_viewWidth = 1;
			_viewHeight = 1;
		}
		
		public function resize(w:int, h:int):void
		{
			var cntrX:int, cntrY:int;
			trace ("Resizing viewing grid");
			// KLUDGE IMPLEMENTATION _REWRITE WHEN HAVE TIME
			// simply erasing old grid
			var row:Array;
			var image:Image
			while (_tiles.length > 0) {
				row = _tiles.pop() as Array;
				while (row.length > 0) {
					image = row.pop() as Image;
					removeChild(image);
				}
			}
			
			// then creating new grid
			var tileWidth:Number = Number(_viewportWidth / (w-1));
			var tileHeight:Number = Number(_viewportHeight / (h-1));
			var tileScaleX:Number = tileWidth / _textures[0].width;
			var tileScaleY:Number = tileHeight / _textures[0].height;
			for (cntrY = 0; cntrY < h; ++cntrY) {
				_tiles[cntrY] = new Array(w);
				for (cntrX = 0; cntrX < w; ++cntrX) {
					image = new Image(_textures[0]);
					image.x = cntrX * tileWidth;
					image.y = cntrY * tileHeight;
					image.scaleX = tileScaleX;
					image.scaleY = tileScaleY;
					addChild(image);
					_tiles[cntrY][cntrX] = image;
				}
			}
			_viewWidth = w;
			_viewHeight = h;
		}
		
// ***** METHODS - ITileMapRenderer Implementation *****
		
		public function setViewportSize(w:int, h:int):void
		{
			_viewportWidth = w;
			_viewportHeight = h;
		}
		
		public function renderLayer(worldView:Rectangle):void
		{
			var cntrX:int, cntrY:int;
			var viewW:int = Math.ceil(worldView.width + 1);
			var viewH:int = Math.ceil(worldView.height + 1);
			var left:int = worldView.x;
			var top:int = worldView.y;
			var w:int = Math.ceil(worldView.width);
			var h:int = Math.ceil(worldView.height);
			var needRedraw:Boolean = _map.mapChanged;
			
			if ( (_viewWidth !=  viewW) || (_viewHeight != viewH) ) {
				resize(viewW, viewH);
				needRedraw = true;
			}
			if ((_oldX != left) || (_oldY != top)) {
				needRedraw = true;
			}
			if (needRedraw)
			{
				// adjust tiles
				var i:Image;
				for (cntrY = 0; cntrY < viewH; ++cntrY) {
					for (cntrX = 0; cntrX < viewW; ++cntrX) {
						i = _tiles[cntrY][cntrX];
						i.texture = _textures[_map.getTile(cntrX + left, cntrY + top)];
					}
				}
				_oldX = left;
				_oldY = top;
			}
			
			// adjust viewport position
			var tileWidth:Number = Number(_viewportWidth / w);
			var tileHeight:Number = Number(_viewportHeight / h);
			x = left * tileWidth - Math.floor(worldView.x * tileWidth);
			y = top * tileHeight - Math.floor(worldView.y * tileHeight);
		}
	}
}