package com.blazinggames.gamejam.layers.starling
{
	import starling.core.starling_internal;
	import starling.display.Image;
	import starling.display.Sprite;
	import starling.textures.Texture;
	
	public class WorldSprite extends Sprite
	{
		public static const RAD_TO_DEGREE:Number = 180 / Math.PI;
		public static const DEGREE_TO_RAD:Number = Math.PI / 180;

		private var _worldX:Number, _worldY:Number;
		private var _radius:Number;
		
		public function WorldSprite(makeColorSprite:uint = 0)
		{
			super();
			_worldX = _worldY = 0;
			_radius = .45;
			if (makeColorSprite > 0) {
				var image = new Image(Texture.fromColor(32,32,makeColorSprite));
				image.x = -16;
				image.y = -16;
				addChild(image);
			}
		}
		
		public static function findAngle(srcX:Number, srcY:Number, 
										 targetX:Number, targetY:Number, 
										 inRads:Boolean = true):Number
		{
			var rads:Number = Math.atan2(targetY - srcY, targetX - srcX);
			if (inRads)
				return rads;
			else
				return rads * RAD_TO_DEGREE;
		}
		
		public static function findSpriteAngle(src:WorldSprite, target:WorldSprite, inRads:Boolean = true):Number
		{
			return findAngle(src.worldX, src.worldY, target.worldX, target.worldY, inRads);	
		}
		
		public static function findDistanceSquared(obj1X:Number, obj1Y:Number, obj2X:Number, obj2Y:Number):Number
		{
			var deltaX:Number = obj1X - obj2X;
			var deltaY:Number = obj1Y - obj2Y;
			return (deltaX * deltaX + deltaY * deltaY);
		}

// ***** METHODS - Getters and Setters *****
		
		public function get worldX():Number
		{
			return _worldX;
		}

		public function set worldX(value:Number):void
		{
			_worldX = value;
		}

		public function get worldY():Number
		{
			return _worldY;
		}

		public function set worldY(value:Number):void
		{
			_worldY = value;
		}

		public function get radius():Number
		{
			return _radius;
		}

		public function set radius(value:Number):void
		{
			_radius = value;
		}


	}
}