package com.blazinggames.gamejam.ui.starling
{
	import starling.display.Image;
	import starling.display.Sprite;
	import starling.text.BitmapFont;
	import starling.text.TextField;
	import starling.textures.Texture;
	import starling.utils.HAlign;
	
	public class Console extends Sprite
	{
		[Embed(source="lcd.gif")]
		public static const LCDTexture:Class;		
		[Embed(source="lcd.xml", mimeType="application/octet-stream")]
		public static const FontXML:Class;
		private static var _font:BitmapFont;

		private var _backdrop:Image;
		private var _lines:Array;
		private var _rows:int;
		private var _cols:int;
		
		public function Console(cols:int, rows:int, charSize:int = 16, backColor:uint=0xFF000000, textColor:uint = 0xFFFFFFFF)
		{
			super();
			if (_font == null) {
				var fontTexture:Texture = Texture.fromBitmap(new LCDTexture());
				var fontXML:XML = XML(new FontXML());
				
				_font = new BitmapFont(fontTexture, fontXML);
				TextField.registerBitmapFont(_font);				
			}
			var w:int = charSize * cols;
			_backdrop = new Image(Texture.fromColor(w, charSize * rows, backColor));
			addChild(_backdrop);
			_rows = rows;
			_cols = cols;
			_lines = new Array(rows);
			for (var cntr:int = 0; cntr < rows; ++cntr) {
				//				_lines[cntr] = new TextField(w, charSize, " ", "_typewriter", charSize-2, textColor);
				_lines[cntr] = new TextField(w, charSize, " ", "LCD", charSize, textColor);
				_lines[cntr].y = cntr * charSize;
				_lines[cntr].hAlign = HAlign.LEFT;
				//_lines[cntr].vAlign = VAlign.BOTTOM;
				addChild(_lines[cntr]);
			}
		}
		
		public function clear():void
		{
			for (var cntr:int = 0; cntr < _rows; ++cntr)
				setLine(cntr, " ");
		}
		
		public function setLine(row:int, text:String):void
		{
			_lines[row].text = text;
		}
		
		public function write(text:String):void
		{
			for (var cntr:int = 1; cntr < _rows; ++cntr)
				_lines[cntr-1].text = _lines[cntr].text;
			_lines[_rows-1].text = text;
		}
		
		public function get rows():int
		{
			return _rows;
		}
		
		public function get columns():int
		{
			return _cols;
		}
	}
}