package com.blazinggames.gamejam.ui.starling
{
	import flash.ui.Keyboard;

	public class ConsolePrompt
	{
		private var _console:Console;
		private var _promptMarker:String;
		private var _blankMarker:String;
		private var _currentSelection:int;
		private var _choices:Array;
		
		public function ConsolePrompt(console:Console, promptMarker:String = "-> ")
		{
			_console = console;
			_promptMarker = promptMarker;
			_blankMarker = " ";
			for (var cntr:int = 1; cntr < _promptMarker.length; ++cntr)
				_blankMarker = _blankMarker + " ";
			_choices = new Array();
			_currentSelection = 0;
		}
		
		public function clear():void
		{
			while (_choices.length > 0)
				_choices.pop();
			_currentSelection = 0;
		}
		
		public function addChoice(choice:String):int
		{
			_choices.push(choice);
			if ((_choices.length - 1) == _currentSelection)
				_console.write(_promptMarker + choice);
			else
				_console.write(_blankMarker + choice);
			return choice.length - 1;
		}
		
		private function redrawConsole():void
		{
			var startLine:int = _console.rows - _choices.length;
			for (var cntr:int = 0; cntr < _choices.length; ++cntr)
				if (cntr == _currentSelection)
					_console.setLine(cntr + startLine, _promptMarker + _choices[cntr]);
				else
					_console.setLine(cntr + startLine, _blankMarker + _choices[cntr]);
		}
		
		public function nextChoice():int
		{
			if ((_currentSelection + 1) < _choices.length) {
				++_currentSelection;
				redrawConsole();
			}
			return _currentSelection;
		}
		
		public function previousChoice():int
		{
			if ((_currentSelection - 1) > -1) {
				--_currentSelection;
				redrawConsole();
			}
			return _currentSelection;
		}
		
		public function getChoice():int
		{
			return _currentSelection;
		}
		
		public function processKey(keycode:int, isDown:Boolean):Boolean
		{
			var hasMadeChoice:Boolean = false;
			switch(keycode) {
				case Keyboard.UP:
					if (isDown)
						previousChoice();
					break;
				case Keyboard.DOWN:
					if (isDown)
						nextChoice();
					break;
				case Keyboard.Z:
				case Keyboard.ENTER:
					if (isDown == false) {
						hasMadeChoice = true;
					}
			}
			return hasMadeChoice;
			
		}
	}
}