package com.blazinggames.gamejam.ui.starling
{
	import flash.geom.Point;
	
	import starling.display.Image;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.textures.Texture;
	
	public class Slider extends Sprite
	{
		// ***** CONSTANTS *****
		
		public static const VALUE_CHANGED_EVENT:String = "ValueChanged";
		
		// ***** VARIABLES *****
		
		private var _backdrop:Image;
		private var _slider:Image;
		private var _startPoint:Point;
		private var _endPoint:Point;
		private var _value:Number;
		
		// ***** CONSTRUCTOR *****
		
		public function Slider(backdrop:Texture, slider:Texture, slideStart:Point, slideStop:Point)
		{
			super();
			_backdrop = new Image(backdrop);
			addChild(_backdrop);
			_slider = new Image(slider);
			_slider.x = slideStart.x;
			_slider.y = slideStart.y;
			_slider.addEventListener(TouchEvent.TOUCH, handleSlider);
			addChild(_slider);
			_startPoint = slideStart;
			_endPoint = slideStop;
		}
		
		private function adjustSlider(p:Point):void
		{
			var distX:Number = _endPoint.x - _startPoint.x;
			var distY:Number = _endPoint.y - _startPoint.y;
			var val:Number;
			
			if (distX > distY) {
				val = (p.x - _startPoint.x) / distX;
			} else {
				val = (p.y - _startPoint.y) / distY;
			}
			val = Math.min(Math.max(0, val), 1);
			
			_slider.x = _startPoint.x + distX * val;
			_slider.y = _startPoint.y + distY * val;
			_value = val;
		}
		
		private function handleSlider(e:TouchEvent):void
		{
			var touch:Touch = e.getTouch(_slider);
			var p:Point;
			
			if (touch == null)
				return;
			if (touch.phase == TouchPhase.MOVED) {
				p = touch.getLocation(this);
				adjustSlider(p);
			} else if (touch.phase == TouchPhase.ENDED)
				dispatchEvent(new Event(VALUE_CHANGED_EVENT))
		}
		
		public function get value():Number
		{
			return _value;
		}
		
		public function set value(value:Number):void
		{
			_value = Math.max(Math.min(1, value), 0);
			var distX:Number = _endPoint.x - _startPoint.x;
			var distY:Number = _endPoint.y - _startPoint.y;
			_slider.x = _startPoint.x + distX * _value;
			_slider.y = _startPoint.y + distY * _value;
		}
	}
}